@@ -31,7 +31,9 @@ class XLIFF:
         self.xliff_version = float(self.xml_root.attrib['version'])
         self.nsmap = self.xml_root.nsmap
 
-        self.translation_units = etree.Element('translation-units')
+    def get_translation_units(self, include_segments_wo_id=True):
+
+        translation_units = etree.Element('translation-units')
 
         if self.xliff_version >= 2.0:
             for translation_unit in self.xml_root.findall('.//unit', self.nsmap):
@@ -44,22 +46,27 @@ class XLIFF:
                     if 'equiv' in _any_child.attrib:
                         _any_child.text = html.unescape(_any_child.attrib['equiv'])
 
-                self.translation_units.append(_translation_unit)
+                translation_units.append(_translation_unit)
         else:
             for translation_unit in self.xml_root.findall('.//trans-unit', self.nsmap):
                 segments = []
-                _g_counter = {}
-                _translation_unit = etree.Element('translation-unit', translation_unit.attrib)
-                if translation_unit.find('seg-source', self.nsmap) is not None:
-                    for source_segment in translation_unit.findall('seg-source//mrk', self.nsmap):
+                if translation_unit.find('seg-source//mrk[@mtype="seg"]', self.nsmap) is not None:
+                    for source_segment in translation_unit.findall('seg-source//mrk[@mtype="seg"]', self.nsmap):
                         target_segment = translation_unit.find('target//mrk[@mid="{0}"]'.format(source_segment.attrib['mid']), self.nsmap)
 
                         segments.append([source_segment, target_segment])
+                elif translation_unit.find('seg-source', self.nsmap) is not None and include_segments_wo_id:
+                    for source_segment in translation_unit.findall('seg-source', self.nsmap):
+                        target_segment = translation_unit.find('target', self.nsmap)
 
-                else:
+                        segments.append([source_segment, target_segment])
+                elif translation_unit.find('source', self.nsmap) is not None and include_segments_wo_id:
                     segments.append([translation_unit.find('source', self.nsmap), translation_unit.find('target', self.nsmap)])
 
+
+                _translation_unit = etree.Element('translation-unit', translation_unit.attrib)
                 for segment in segments:
+
                     _segment = etree.SubElement(_translation_unit, 'segment', {'id': segment[0].attrib.get('mid', 'N/A')})
 
                     _source = deepcopy(segment[0])
@@ -84,14 +91,11 @@ class XLIFF:
                             elif _any_child.tag.startswith('e'):
                                 _any_child.text = '</{0}-{1}>'.format(etree.QName(_any_child).localname[1:], _any_child.attrib.get('id', 'N/A'))
                             elif _any_child.tag.endswith('g'):
-                                if _any_child.attrib['id'] not in _g_counter:
-                                    _g_counter[_any_child.attrib['id']] = str(len(_g_counter) + 1)
                                 _b_g_tag = etree.Element('g', _any_child.attrib)
-                                _b_g_tag.attrib['no'] = _g_counter[_any_child.attrib['id']]
                                 _e_g_tag = deepcopy(_b_g_tag)
 
-                                _b_g_tag.text = '<g-{0}>'.format(_g_counter[_any_child.attrib['id']])
-                                _e_g_tag.text = '</g-{0}>'.format(_g_counter[_any_child.attrib['id']])
+                                _b_g_tag.text = '<g-{0}>'.format(_any_child.attrib['id'])
+                                _e_g_tag.text = '</g-{0}>'.format(_any_child.attrib['id'])
 
                                 _parent = _any_child.getparent()
                                 _parent.replace(_any_child, _b_g_tag)
@@ -105,7 +109,11 @@ class XLIFF:
                             else:
                                 _any_child.text = '<{0}-{1}/>'.format(etree.QName(_any_child).localname, _any_child.attrib.get('id', 'N/A'))
 
-                self.translation_units.append(_translation_unit)
+                translation_units.append(_translation_unit)
+
+        etree.cleanup_namespaces(translation_units)
+
+        return translation_units
 
     def merge_segments(self, *args):
         raise TypeError('This function is available for the kxliff.KXLIFF class only.')
@@ -224,25 +232,6 @@ class XLIFF:
 
         _target_segment = deepcopy(target_segment)
 
-        translation_unit = self.translation_units.find('translation-unit[@id="{0}"]'.format(tu_no))
-
-        if segment_no is not None:
-            segment = translation_unit.find('segment[@id="{0}"]'.format(segment_no), self.nsmap)
-            if segment is None:
-                segment = translation_unit.find('segment[@id="{0}"]'.format(segment_no))
-        else:
-            segment = translation_unit.findall('segment', self.nsmap)[0]
-            if segment is None:
-                segment = translation_unit.findall('segment')[0]
-
-        target = segment.find('target', self.nsmap)
-        if target is None:
-            target = segment.find('target')
-        if target is None:
-            segment.append(target_segment)
-        else:
-            segment[segment.index(target)] = target_segment
-
         if self.xliff_version >= 2.0:
             _translation_unit = self.xml_root.find('.//unit[@id="{0}"]'.format(tu_no), self.nsmap)
 
